(function() {
  var DMStore, Vue, VuePlugin, hs, uuid,
    bind = function(fn, me){ return function(){ return fn.apply(me, arguments); }; },
    hasProp = {}.hasOwnProperty;

  uuid = require('uuid/v4');

  if ((typeof window !== "undefined" && window !== null ? window.Vue : void 0) == null) {
    Vue = require('vue');
  }

  hs = require('./helpers');

  DMStore = (function() {
    function DMStore(name1) {
      this.name = name1;
      this.mutate_value = bind(this.mutate_value, this);
      this.mutate_collection = bind(this.mutate_collection, this);
      this.init_value = bind(this.init_value, this);
      this.init_collection = bind(this.init_collection, this);
      this._mutate = bind(this._mutate, this);
    }

    DMStore.prototype.attach = function(state1) {
      this.state = state1;
    };

    DMStore.prototype._get_deep = function(object, keypath) {
      var i, index, key, keys, len, sub;
      keys = keypath.split('.');
      sub = object;
      for (index = i = 0, len = keys.length; i < len; index = ++i) {
        key = keys[index];
        if (index === keys.length - 1) {
          return sub[key];
        } else {
          if (sub[key] != null) {
            sub = sub[key];
          } else {
            return;
          }
        }
      }
    };

    DMStore.prototype._set_deep = function(object, keypath, value) {
      var i, index, key, keys, len, sub;
      keys = keypath.split('.');
      sub = object;
      for (index = i = 0, len = keys.length; i < len; index = ++i) {
        key = keys[index];
        if (index === keys.length - 1) {
          sub[key] = value;
          return true;
        } else {
          if (sub[key] != null) {
            sub = sub[key];
          } else {
            return false;
          }
        }
      }
    };

    DMStore.prototype._mutate = function(selector, value, track_changes) {
      var set;
      set = this._set_deep(this.state, selector, value);
      if (set) {
        return this.state._dirty = hs["typeof"](track_changes) === 'boolean' ? track_changes : true;
      } else {
        throw new Error("No such property '" + selector + "' in <" + this.name + "> component state object");
      }
    };

    DMStore.prototype.init_collection = function(selector, data, key) {
      if (key == null) {
        key = '_id';
      }
      return this.mutate_collection(selector, data, key, false);
    };

    DMStore.prototype.init_value = function(selector, data) {
      return this.mutate_value(selector, data, false);
    };

    DMStore.prototype.mutate_collection = function(selector, data, key, track_changes) {
      var i, item, len, m;
      if (key == null) {
        key = '_id';
      }
      m = new Map;
      for (i = 0, len = data.length; i < len; i++) {
        item = data[i];
        m.set(item[key], item);
      }
      return this._mutate(selector, m, track_changes);
    };

    DMStore.prototype.mutate_value = function(selector, data, track_changes) {
      return this._mutate(selector, data, track_changes);
    };

    return DMStore;

  })();

  VuePlugin = (function() {
    function VuePlugin() {}

    VuePlugin.prototype.install = function(Vue, options) {
      return Vue.mixin({
        data: function() {
          return {
            _dmstate: {},
            store: {},
            dmstore_uuid: uuid()
          };
        },
        directives: {
          'state-model': {
            bind: function(el, binding, vnode) {
              var component_state, event_type, keypath, state, store, value_key;
              keypath = binding.expression;
              store = vnode.context.store.app;
              state = vnode.context._dmstate;
              component_state = vnode.context.state;
              if (el.tagName.toLowerCase() === 'input' && el.type.toLowerCase() === 'text') {
                event_type = 'input';
                value_key = 'value';
              } else {
                event_type = 'change';
                value_key = 'checked';
              }
              el[value_key] = store._get_deep(vnode.context, keypath);
              return el.addEventListener(event_type, function(event) {
                var dot, value;
                value = event.target[value_key];
                dot = keypath.indexOf('.');
                if (dot > -1) {
                  keypath = keypath.substring(dot + 1);
                }
                store.mutate_value(keypath, value);
                return store._set_deep(component_state, keypath, value);
              });
            }
          }
        },
        methods: {
          clean: function() {
            var ref;
            return (ref = this.$root.store.state[this.dmstore_uuid]) != null ? ref._dirty = false : void 0;
          }
        },
        created: function() {
          var dm, name;
          if ((this.$parent != null) && (this.state != null)) {
            name = this.$options.name || 'unnamed';
            dm = new DMStore(name);
            this.$root.store.state[this.dmstore_uuid] = {
              _dirty: false
            };
            dm.attach(this.$root.store.state[this.dmstore_uuid]);
            return Vue.set(this.store, 'app', dm);
          } else {
            return Vue.set(this.store, 'state', {});
          }
        },
        destroyed: function() {
          if (this.$parent == null) {
            return;
          }
          return delete this.$root.store.state[this.dmstore_uuid];
        },
        mounted: function() {
          var k, ref, v;
          if (this.$parent == null) {
            return;
          }
          if (this.state == null) {
            return;
          }
          ref = this.state;
          for (k in ref) {
            if (!hasProp.call(ref, k)) continue;
            v = ref[k];
            this.$root.store.state[this.dmstore_uuid][k] = v;
            this.$root.store.state[this.dmstore_uuid]._dirty = false;
          }
          return this._dmstate = this.$root.store.state[this.dmstore_uuid];
        }
      });
    };

    return VuePlugin;

  })();

  module.exports = VuePlugin;

}).call(this);
